<?php
declare(strict_types=1);

namespace CakeProducts\Model\Table;

use CakeProducts\Model\Entity\ProductCategoryVariantOption;
use CakeProducts\Model\Table\ProductCategoryVariantsTable;
use Cake\Datasource\EntityInterface;
use Cake\Datasource\ResultSetInterface;
use Cake\ORM\Association\BelongsTo;
use Cake\ORM\Behavior\TimestampBehavior;
use Cake\ORM\Query\SelectQuery;
use Cake\ORM\RulesChecker;
use Cake\ORM\Table;
use Cake\Validation\Validator;
use Closure;
use Psr\SimpleCache\CacheInterface;

/**
 * ProductCategoryVariantOptions Model
 *
 * @property ProductCategoryVariantsTable&BelongsTo $ProductCategoryVariants
 *
 * @method ProductCategoryVariantOption newEmptyEntity()
 * @method ProductCategoryVariantOption newEntity(array $data, array $options = [])
 * @method array<ProductCategoryVariantOption> newEntities(array $data, array $options = [])
 * @method ProductCategoryVariantOption get(mixed $primaryKey, array|string $finder = 'all', CacheInterface|string|null $cache = null, Closure|string|null $cacheKey = null, mixed ...$args)
 * @method ProductCategoryVariantOption findOrCreate($search, ?callable $callback = null, array $options = [])
 * @method ProductCategoryVariantOption patchEntity(EntityInterface $entity, array $data, array $options = [])
 * @method array<ProductCategoryVariantOption> patchEntities(iterable $entities, array $data, array $options = [])
 * @method ProductCategoryVariantOption|false save(EntityInterface $entity, array $options = [])
 * @method ProductCategoryVariantOption saveOrFail(EntityInterface $entity, array $options = [])
 * @method iterable<ProductCategoryVariantOption>|ResultSetInterface<ProductCategoryVariantOption>|false saveMany(iterable $entities, array $options = [])
 * @method iterable<ProductCategoryVariantOption>|ResultSetInterface<ProductCategoryVariantOption> saveManyOrFail(iterable $entities, array $options = [])
 * @method iterable<ProductCategoryVariantOption>|ResultSetInterface<ProductCategoryVariantOption>|false deleteMany(iterable $entities, array $options = [])
 * @method iterable<ProductCategoryVariantOption>|ResultSetInterface<ProductCategoryVariantOption> deleteManyOrFail(iterable $entities, array $options = [])
 *
 * @mixin TimestampBehavior
 */
class ProductCategoryVariantOptionsTable extends Table
{
    /**
     * Initialize method
     *
     * @param array<string, mixed> $config The configuration for the Table.
     * @return void
     */
    public function initialize(array $config): void
    {
        parent::initialize($config);

        $this->setTable('product_category_variant_options');
        $this->setDisplayField('variant_value');
        $this->setPrimaryKey('id');

        $this->addBehavior('Timestamp');

        $this->belongsTo('ProductCategoryVariants', [
            'foreignKey' => 'product_category_variant_id',
            'joinType' => 'INNER',
        ]);
    }

    /**
     * Default validation rules.
     *
     * @param Validator $validator Validator instance.
     * @return Validator
     */
    public function validationDefault(Validator $validator): Validator
    {
        $validator
            ->uuid('product_category_variant_id')
            ->notEmptyString('product_category_variant_id');

        $validator
            ->scalar('variant_value')
            ->maxLength('variant_value', 255)
            ->requirePresence('variant_value', 'create')
            ->notEmptyString('variant_value');

        $validator
            ->scalar('variant_label')
            ->maxLength('variant_label', 255)
            ->allowEmptyString('variant_label');

        $validator
            ->dateTime('deleted')
            ->allowEmptyDateTime('deleted');

        return $validator;
    }

    /**
     * Returns a rules checker object that will be used for validating
     * application integrity.
     *
     * @param RulesChecker $rules The rules object to be modified.
     * @return RulesChecker
     */
    public function buildRules(RulesChecker $rules): RulesChecker
    {
        $rules->add($rules->existsIn(['product_category_variant_id'], 'ProductCategoryVariants'), ['errorField' => 'product_category_variant_id']);

        return $rules;
    }
}
